SystemOrganization addCategory: #Rest!
SystemOrganization addCategory: #'Rest-Tests'!
SystemOrganization addCategory: #'Rest-Handlers'!
SystemOrganization addCategory: #'Rest-Services'!

HttpService subclass: #RestHttpService
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Services'!

RestHttpService class
	instanceVariableNames: 'service'!

!RestHttpService commentStamp: 'mt 2/16/2009 14:56' prior: 0!
I am a restful http service. I am primarily used for the XP integration platform project. (Package: 'Xpf')!

RestHttpService class
	instanceVariableNames: 'service'!

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/16/2009 15:14'!
isRunning
	^service isNil not and: [service isRunning]! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/16/2009 14:57'!
port
	^8082! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/16/2009 15:10'!
service
	^service! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/16/2009 15:04'!
start
	"Start exactly one http service on port 8082. This service will be closed first if you call this message twice."

	self stop.
	service := (RestHttpService on: self port)
		start;
		yourself.! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/16/2009 14:52'!
stop
	"Shutdown the server. Does nothing if server is not running."

	service ifNotNil:
		[service unregister.
		service := nil].! !

!RestHttpService methodsFor: 'initialization' stamp: 'mt 2/2/2009 10:12'!
initialize
	super initialize.
	handlers := OrderedCollection new	"needed for priorities"! !

!RestHttpService methodsFor: 'serving' stamp: 'cs 2/15/2009 16:07'!
installHandler: aRequestHandlerClass on: aPathString
	(handlers anySatisfy: [:ass | ass key = aPathString ])
		ifTrue: [^ false].
	handlers add: aPathString -> aRequestHandlerClass! !

!RestHttpService methodsFor: 'serving' stamp: 'mt 2/16/2009 18:02'!
processHttpRequest: request
	
	| activeHandler path |
	request isPutRequest ifTrue: [Transcript show: request fields].
	activeHandler := (handlers detect: [:ass| 
		path := ass key.
		request url includesSubString: path
		]) value.
	^(activeHandler new) process: request! !

Object subclass: #RestHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Handlers'!

!RestHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 15:18'!
process: anHttpRequest
	"Handles an http request. Should answer with an http response."
	
	self subclassResponsibility! !

RestHttpHandler subclass: #RestPerformHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Handlers'!

!RestPerformHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 15:25'!
process: anHttpRequest
	"Takes first element in path as classname and sends message from third path element to that class."

	|pc class selector result|
	pc := (URI fromString: anHttpRequest url) pathComponents.
	class := pc first asSymbol.
	selector := pc second asSymbol.
	
	Smalltalk at: class ifPresent: [:c | 
		(c class allSelectors includes: selector) ifTrue: [
			result := c perform: selector]].

	^ HttpResponse fromString: result printString.! !

RestHttpHandler subclass: #RestResourceHttpHandler
	instanceVariableNames: 'resources'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Handlers'!

!RestResourceHttpHandler methodsFor: 'http-post-helper' stamp: 'mt 2/16/2009 17:18'!
create: resClass from: fields
	"Works only with a new project or a new user."
	
	| entity |
	entity := resClass new.
	fields keysAndValuesDo: [:key :value |
		entity instVarNamed: key asString put: value first].
	resClass perform: ('add', (self resolveName: resClass) capitalized, ':') asSymbol with: entity.
	^ HttpResponse status: #ok contents: entity id asString! !

!RestResourceHttpHandler methodsFor: 'http-delete-helper' stamp: 'mt 2/16/2009 17:36'!
delete: resClass id: id
	"Delete a resource with the specified identifier. Works only with projects and users."

	| entity |
	entity := resClass all
		detect: [:each | each id = id]
		ifNone: [^ self responseForNotFound: resClass id: id].
	resClass
		perform: ('delete', (self resolveName: resClass) capitalized, ':') asSymbol
		with: entity.
	^ HttpResponse status: #ok contents: 'deleted'.! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'mt 2/16/2009 16:35'!
getRequestChainFor: aPathComponentList
	"Create a list of associations for the request. That list contains resource classes and their requested ids."
	
	| chain resClass idNumber |
	chain := OrderedCollection new.
	aPathComponentList size odd ifTrue: [aPathComponentList add: 'nil'].
	aPathComponentList pairsDo: [:resName :idName | 
		resClass := self resolveResource: resName ifAbsent: [nil].
		idName = 'nil'
			ifTrue: [idNumber := nil]
			ifFalse: [idNumber := idName asNumber].
		resClass ifNotNil: [chain add: (resClass -> idNumber)]].
	^ chain! !

!RestResourceHttpHandler methodsFor: 'initialization' stamp: 'mt 2/18/2009 14:57'!
initialize
	
	super initialize.
	resources := Dictionary new! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'cs 2/17/2009 18:37'!
process: anHttpRequest 
	"Create a resource chain from the path of the http request. Fail if that chain is empty. Redirect processing according to used verb (POST, PUT, GET, DELETE)."

	| res |
	res := self getRequestChainFor: (URI fromString: anHttpRequest url) pathComponents.
	res isEmpty ifTrue: [^ self responseForFailure: anHttpRequest].
	
	anHttpRequest isPostRequest ifTrue: [^ self processPost: res fields: anHttpRequest postFields].
	anHttpRequest isGetRequest ifTrue: [^ self processGet: res].
	anHttpRequest isPutRequest ifTrue: [^ self processPut: res fields: anHttpRequest putFields].
	anHttpRequest isDeleteRequest ifTrue: [^ self processDelete: res].! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 16:59'!
processDelete: aResourceChain 

	|resClass id|
	resClass := aResourceChain first key.
	id := aResourceChain first value.
	^ self delete: resClass id: id! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 16:57'!
processGet: aResourceChain 

	|resClass id|
	resClass := aResourceChain first key.
	id := aResourceChain first value.
	
	id isNil
		ifTrue: [^ self read: resClass]
		ifFalse:[^ self read: resClass id: id].! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 16:56'!
processPost: aResourceChain fields: someFields 
	
	|resClass|
	resClass := aResourceChain first key.
	^ self create: resClass from: someFields! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 16:58'!
processPut: aResourceChain fields: someFields 

	|resClass id|
	resClass := aResourceChain first key.
	id := aResourceChain first value.
	^ self update: resClass from: someFields id: id! !

!RestResourceHttpHandler methodsFor: 'http-get-helper' stamp: 'mt 2/16/2009 17:24'!
read: resClass
	"Read information about all entities of a given class. Works only for projects or users."
	
	| response |
	response := resClass all collect: [:entity|
		Dictionary newFrom: (
			resClass allInstVarNames collect: [:name| name -> (entity instVarNamed: name)])].
	^ HttpResponse status: #ok contents: (Json render: response).! !

!RestResourceHttpHandler methodsFor: 'http-get-helper' stamp: 'mt 2/16/2009 17:36'!
read: resClass id: id
	"Read information about one special resource with the given id. Works only for projects and users."

	| entity response |
	entity := resClass all
		detect: [:each | each id = id]
		ifNone: [^ self responseForNotFound: resClass id: id].
	response := Dictionary new.
	XpfProject allInstVarNames do:
		[:name| response at: name put: (entity instVarNamed: name)].
	^ HttpResponse status: #ok contents: (Json render: response).! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'mt 2/16/2009 17:35'!
resolveName: aClass
	"Inverted lookup in the pool of registered resourcen."

	resources keysAndValuesDo: [:key :value |
		value = aClass ifTrue: [^ key]].
	^ nil.! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'mt 2/16/2009 16:45'!
resolveResource: aResourceName ifAbsent: aBlock
	"Try to resolve the given resource name to a key in the registered resource pool. Plural will be converted to singular: userstories -> userstory, projects -> project. The resource name has to be lowercase."

	| resourceKey |
	resourceKey := aResourceName.
	(resourceKey endsWith: 'ies')
		ifTrue:[resourceKey := (resourceKey truncateTo: resourceKey size - 3), 'y'].
	(resourceKey endsWith: 's')
		ifTrue:[resourceKey := resourceKey truncateTo: resourceKey size - 1].
		
	^ resources at: resourceKey ifAbsent: aBlock.! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'mt 2/16/2009 17:37'!
responseForFailure: anHttpRequest 
	^ nil asHttpResponseTo: anHttpRequest! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'mt 2/16/2009 17:35'!
responseForNotFound: resource id: id
	^ HttpResponse
		status: #notFound
		contents:
			(self resolveName: resource capitalized),
			' with id ', id asString,
			' not found.'! !

!RestResourceHttpHandler methodsFor: 'http-put-helper' stamp: 'mt 2/16/2009 17:36'!
update: resClass from: fields id: id
	"Works only with projects and users."

	| entity |
	entity := resClass all
		detect: [:each | each id = id]
		ifNone: [^ self responseForNotFound: resClass id: id].
	fields keysAndValuesDo: [:key :value |
		entity instVarNamed: key asString put: value first].
	^ HttpResponse status: #ok contents: 'updated'! !

RestResourceHttpHandler subclass: #RestXpfResourceHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Handlers'!

!RestXpfResourceHttpHandler methodsFor: 'initialization' stamp: 'mt 2/18/2009 14:57'!
initialize
	super initialize.
	self registerXpfEntityClasses.! !

!RestXpfResourceHttpHandler methodsFor: 'initialization' stamp: 'mt 2/18/2009 14:57'!
registerXpfEntityClasses
	"Add all subclasses of XpfEntity to the pool of accessible resources. Example: 'XpfUserStory' will have the key 'userstory' to lookup the entity class."

	XpfEntity allSubclasses do: [:aClass |
		resources
			at: (aClass name copyReplaceAll: 'Xpf' with: '') asLowercase
			put: aClass]! !

Object subclass: #RestLogService
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Services'!

!RestLogService commentStamp: 'mt 2/16/2009 14:53' prior: 0!
You can use me to log messages whenever you want. I will decide what to do with that messages. Normally they will be shown on the Transcript.!

!RestLogService class methodsFor: 'service' stamp: 'cs 2/7/2009 14:20'!
log: aCollection
	Transcript show: aCollection first asString; cr.! !

!XpfEntity methodsFor: '*rest' stamp: 'cs 2/16/2009 00:40'!
jsonWriteOn: aStream
	self id jsonWriteOn: aStream! !

!Dictionary methodsFor: '*rest' stamp: 'mt 2/16/2009 10:28'!
keysAndValuesCollect: aBlock 
	| newDict |
	newDict := Dictionary new.
	self keysAndValuesDo: [:key :value | newDict add: (aBlock value: key value: value)].
	^ newDict! !

!Collection methodsFor: '*rest' stamp: 'cs 2/16/2009 00:38'!
jsonWriteOn: aStream
	| needComma |
	needComma := false.
	aStream nextPut: $[.
	self do: [:v |
		needComma
			ifTrue: [ aStream nextPutAll: ', ' ]
			ifFalse: [ needComma := true ].
		v jsonWriteOn: aStream.
	].
	aStream nextPut: $].! !

!HttpRequest methodsFor: '*rest' stamp: 'mt 2/18/2009 14:21'!
putFields
	"Answer the multipart fields for a post request (if in fact this is a
	PUT request"

	^self propertyAt: #putFields ifAbsentPut: [
		" parse PUT fields in the body of request, if it's standard form data "

		(self isPutRequest and: [ 
			self contentType = MIMEDocument contentTypeFormData 
		]) ifTrue: [
			self class
				decodeUrlEncodedForm: (stream next: self contentLength)
				multipleValues: true.
		] ifFalse: [
			Dictionary new
		].
	]! !

!HTTPSocket class methodsFor: '*rest-tests' stamp: 'cs 2/15/2009 15:47'!
httpDelete: t1
	| t5 |
	t5 := self
				httpDeleteDocument: t1
				args: nil
				accept: '*/*'
				request: ''.
	t5 isString
		ifTrue: [^ t5].
	^ (RWBinaryOrTextStream with: t5 content) reset! !

!HTTPSocket class methodsFor: '*rest-tests' stamp: 'cs 2/15/2009 15:44'!
httpDeleteDocument: t1 args: t2 accept: t3 request: t4 
	| t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16 t17 t18 t19 t20 |
	Socket initializeNetwork.
	t11 := (t1 asLowercase beginsWith: 'http://')
				ifTrue: [t1 copyFrom: 8 to: t1 size]
				ifFalse: [t1].
	t11 := t11 copyUpTo: $#.
	t5 := t11 copyUpTo: $/.
	t12 := t11 copyFrom: t5 size + 1 to: t11 size.
	(t5 includes: $:)
		ifTrue: [t16 := t5 indexOf: $:.
			t7 := (t5 copyFrom: t16 + 1 to: t5 size) asNumber.
			t5 := t5 copyFrom: 1 to: t16 - 1]
		ifFalse: [t7 := self defaultPort].
	t12 size = 0
		ifTrue: [t12 := '/'].
	t2
		ifNotNil: [t12 := t12
						, (self argString: t2)].
	(self shouldUseProxy: t5)
		ifTrue: [t12 := 'http://' , t5 , ':' , t7 printString , t12.
			t17 := self httpProxyServer.
			t18 := self httpProxyPort]
		ifFalse: [t17 := t5.
			t18 := t7].
	t6 := NetNameResolver addressForName: t17 timeout: 20.
	t6
		ifNil: [^ 'Could not resolve the server named: ' , t17].
	3
		timesRepeat: [t8 := HTTPSocket new.
			t8 connectTo: t6 port: t18.
			(t8
					waitForConnectionUntil: (self deadlineSecs: 30))
				ifFalse: [Socket deadServer: t17.
					t8 destroy.
					^ 'Server ' , t17 , ' is not responding'].
			t8 sendCommand: 'DELETE ' , t12 , ' HTTP/1.0' , CrLf
					, (t3
							ifNil: ['']
							ifNotNil: ['ACCEPT: ' , t3 , CrLf]) , 'ACCEPT: text/html' , CrLf , HTTPBlabEmail , t4 , self userAgentString , CrLf , 'Host: ' , t5 , ':' , t7 printString , CrLf.
			t13 := t8
						getResponseUpTo: CrLf , CrLf
						ignoring: (String with: CR).
			t9 := t13 at: 1.
			t14 := t13 at: 3.
			t9 isEmpty
				ifTrue: [t15 := 'server aborted early']
				ifFalse: [t8 header: t9.
					t10 := t8 getHeader: 'content-length'.
					t10
						ifNotNil: [t10 := t10 asNumber].
					t19 := t8 getHeader: 'content-type'.
					t8 responseCode first = $3
						ifTrue: [t20 := t8 getHeader: 'location'.
							t20
								ifNotNil: [Transcript show: 'redirecting to ' , t20;
										 cr.
									t8 destroy.
									t20 := self
												expandUrl: t20
												ip: t6
												port: t18.
									^ self
										httpGetDocument: t20
										args: t2
										accept: t3
										request: t4]].
					t15 := t8 getRestOfBuffer: t14 totalLength: t10.
					t8 responseCode first = $4
						ifTrue: [^ t9 , t15 contents].
					nil].
			t8 destroy.
			t15 class ~~ String
				ifTrue: [^ MIMEDocument
						contentType: t19
						content: t15 contents
						url: t1].
			t15 = 'server aborted early'
				ifTrue: [^ t15].
			nil].
	{'HTTPSocket class>>httpGetDocument:args:accept:request:'. t15. t1} inspect.
	^ 'some other bad thing happened!!'! !

!HTTPSocket class methodsFor: '*rest-tests' stamp: 'cs 2/15/2009 15:50'!
httpPut: t1 args: t2
	| t4 |
	t4 := self
				httpPutDocument: t1
				args: t2
				accept: '*/*'
				request: ''.
	t4 isString
		ifTrue: [^ t4].
	^ RWBinaryOrTextStream with: t4 content! !

!HTTPSocket class methodsFor: '*rest-tests' stamp: 'cs 2/16/2009 01:42'!
httpPutDocument: t1 args: t2 accept: t3 request: t4 
	| t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 |
	Socket initializeNetwork.
	t14 := Url absoluteFromText: t1.
	t8 := t14 fullPath.
	t15 := t2
				ifNil: ['']
				ifNotNil: [t15 := self argString: t2.
					t15 first = $?
						ifTrue: [t15 := t15 copyFrom: 2 to: t15 size]].
	t5 := HTTPSocket new.
	t5 := self
				initHTTPSocket: t14
				wait: (self deadlineSecs: 30)
				ifError: [:t16 | ^ t16].
	t5 sendCommand: 'PUT ' , t8 , ' HTTP/1.0' , CrLf
			, (t3
					ifNil: ['']
					ifNotNil: ['ACCEPT: ' , t3 , CrLf]) , 'ACCEPT: text/html' , CrLf , HTTPProxyCredentials , HTTPBlabEmail , t4 , self userAgentString , CrLf , 'Content-type: application/x-www-form-urlencoded' , CrLf , 'Content-length: ' , t15 size printString , CrLf , 'Host: ' , t14 authority , CrLf.
	t5 sendCommand: t15.
	t9 := t5
				getResponseUpTo: CrLf , CrLf
				ignoring: (String with: CR).
	t6 := t9 at: 1.
	t10 := t9 at: 3.
	t5 header: t6.
	t7 := t5 getHeader: 'content-length'.
	t7
		ifNotNil: [t7 := t7 asNumber].
	t12 := t5 getHeader: 'content-type'.
	t5 responseCode first = $3
		ifTrue: [t13 := t5 getHeader: 'location'.
			t13
				ifNotNil: [t13 := (t14 newFromRelativeText: t13) asString.
					self flag: #refactor.
					t5 destroy.
					^ self httpGetDocument: t13 accept: t3]].
	t11 := t5 getRestOfBuffer: t10 totalLength: t7.
	t5 responseCode = '401'
		ifTrue: [^ t6 , t11 contents].
	t5 destroy.
	^ MIMEDocument
		contentType: t12
		content: t11 contents
		url: t1! !

TestCase subclass: #RestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests'!

RestTest subclass: #RestHttpServiceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests'!

!RestHttpServiceTest methodsFor: 'as yet unclassified' stamp: 'mt 2/17/2009 16:22'!
setUp
! !

!RestHttpServiceTest methodsFor: 'as yet unclassified' stamp: 'mt 2/17/2009 16:22'!
testStartAndStop
	
	RestHttpService start.
	(Delay forMilliseconds: 100) wait.
	
	self should: (RestHttpService service isKindOf: RestHttpService).
	self should: RestHttpService isRunning.
	
	RestHttpService stop.
	(Delay forMilliseconds: 100) wait.
	
	self should: (RestHttpService service == nil).
	self shouldnt: RestHttpService isRunning.! !

RestTest subclass: #RestPerformTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests'!

!RestPerformTest methodsFor: 'running' stamp: 'mt 2/16/2009 15:24'!
setUp
	super setUp.
	RestHttpService service 
		installHandler: RestPerformHttpHandler
		on: '/'! !

!RestPerformTest methodsFor: 'tests' stamp: 'mt 2/16/2009 15:24'!
testPort
	| response |
	response := self get: '/RestHttpService/port'.
	self should: response contents = RestHttpService port asString.! !

RestTest subclass: #RestResourceTest
	instanceVariableNames: 'testProject testUser testTask'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests'!

!RestResourceTest methodsFor: 'running' stamp: 'mt 2/18/2009 15:30'!
setUp
	"Install a handler for http service. Add a test project to the database."
	
	| testStory |
	super setUp.
	RestHttpService service 
		installHandler: RestResourceHttpHandler
		on: '/'.
		
	testProject := XpfProject new title: 'Test Project'; yourself.
	testUser := XpfUser new login: 'marcel.taeumel'; yourself.
	testStory := XpfUserStory new	title: 'Test UserStory'; yourself.
	testTask := XpfTask new title: 'Test Task'; explanation: 'This is a test task.'; timeEstimated: 12; timeActual: 15; yourself.
	
	XpfProject addProject: testProject.
	XpfUser addUser: testUser.
	testProject attachUserStory: testStory.
	testStory attachTask: testTask.
	"testTask attachDeveloper: testUser."
	! !

!RestResourceTest methodsFor: 'running' stamp: 'cs 2/16/2009 01:29'!
tearDown
	super tearDown.
	(XpfProject all includes: testProject)
		ifTrue: [XpfProject deleteProject: testProject]! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/18/2009 15:00'!
testDeleteOneProject
	"Try to delete a project via http DELETE request. This operation should be idempotent. Verify this by looking into the database directly."

	| projectCount response id |
	projectCount := XpfProject all size.
	id := testProject id.
	
	2 timesRepeat: [
		response := self delete: 'projects/', id asString.
	
		self should: response statusCode = 200.
		self should: XpfProject all size = (projectCount - 1).
		self shouldnt:	(XpfProject all anySatisfy:
				[:aProject | aProject id = id])]! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/18/2009 15:00'!
testDeleteOneTask
	"Try to delete a task from testProject via http DELETE request. This operation should be idempotent. Verify this by looking into the database directly."

	| taskCount response id |
	taskCount := testProject tasks size.
	id := testTask id.
	
	2 timesRepeat: [
		response := self delete: ('projects/{1}/tasks/{2}' format: {testProject id. id}).
	
		self should: response statusCode = 200.
		self should: testProject tasks size = (taskCount - 1).
		self shouldnt:	(testProject tasks anySatisfy:
				[:task | task id = id])]! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/18/2009 14:59'!
testGetAllProjects
	"Try to get a list with rest urls to all projects via http GET request. Verify this by looking into the database directly."

	| response restUrl |
	response := self get: 'projects'.
	response := Json readFrom: (ReadStream on: response).
	
	self should: XpfProject all size = response size.
	
	XpfProject all do: [:project |
		restUrl := '/projects/{1}' format:  {project id}.
		self should: (response includes: restUrl)].! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/18/2009 14:59'!
testGetAllTasks
	"Try to get a list with rest urls to all tasks of testProject via http GET request. Verify this by looking into the database directly."

	| response restUrl |
	response := self get: ('projects/{1}/tasks' format: {testProject id}).
	response := Json readFrom: (ReadStream on: response).
	
	self should: testProject tasks size = response size.
	
	testProject tasks do: [:task |
		restUrl := '/projects/{1}/tasks/{2}' format:  {testProject id. task id}.
		self should: (response includes: restUrl)].! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/18/2009 15:01'!
testGetOneProject
	"Try to get information about a project via http GET request. Verify this by looking into the database directly."

	| response restUrl |	
	response := self get: 'projects/', testProject id asString.
	response := Json readFrom: (ReadStream on: response).
	self should: testProject title = (response at: 'title').
	self should: testProject id = (response at: 'id').
	
	self should: testProject userStories size = (response at: 'userStories') size.
	self should: testProject tasks size = (response at: 'tasks') size.	
	self should: testProject developers size = (response at: 'developers') size.	
	
	testProject userStories do: [:story |
		restUrl := '/projects/{1}/userstories/{2}' format:  {testProject id. story id}.
		self should: ((response at: 'userStories') includes: restUrl)].

	testProject tasks do: [:task |
		restUrl := '/projects/{1}/tasks/{2}' format:  {testProject id. task id}.
		self should: ((response at: 'tasks') includes: restUrl)].
	
	testProject developers do: [:developer |
		restUrl := '/users/{1}' format:  {developer id}.
		self should: ((response at: 'developer') includes: restUrl)].! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/18/2009 15:01'!
testGetOneTask
	"Try to get information about a task from testProject via http GET request. Verify this by looking into the database directly."

	| response restUrl |	
	response := self get: ('projects/{1}/tasks/{2}' format: {testProject id. testTask id}).
	response := Json readFrom: (ReadStream on: response).
	self should: testTask title = (response at: 'title').
	self should: testTask id = (response at: 'id').
	self should: testTask explanation = (response at: 'explanation').
	self should: testTask timeEstimated = (response at: 'timeEstimated').
	self should: testTask timeActual = (response at: 'timeActual').
	self should: ('/projects/', testTask project id) = (response at: 'project').
	
	self should: testTask userStories size = (response at: 'userStories') size.
	self should: testTask developers size = (response at: 'developers') size.	
	
	testTask userStories do: [:story |
		restUrl := '/projects/{1}/userstories/{2}' format:  {testTask id. story id}.
		self should: ((response at: 'userStories') includes: restUrl)].
	
	testTask developers do: [:developer |
		restUrl := '/users/{1}' format:  {developer id}.
		self should: ((response at: 'developer') includes: restUrl)].! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/18/2009 15:37'!
testPostAddProject
	"Try to create a new project via http POST request. Verify this by looking into the database directly."

	| response projectCount newProject |
	projectCount := XpfProject all size.
	response := self
		post: 'projects/addproject'
		fields: (Dictionary with: 'data' -> (self jsonDataFrom: {'title' -> 'new test project'})).

	self should: XpfProject all size = (projectCount + 1).
	newProject := XpfProject all detect:
		[:aProject | aProject id = (response at: 'value') asURI pathComponents last asNumber].
	self should: newProject title = 'new test project'.
	self should: (response at: 'value') = ('/projects/{1}' format: {newProject id})! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/18/2009 15:27'!
testPostAddTask
	"Try to create a new task to testProject via http POST request. Verify this by looking into the database directly."
	
	| response tasksCount newTask |
	tasksCount := testProject tasks size.
	response := self
		post: ('projects/{1}/addtask' format: {testProject id})
		fields: (Dictionary with: 'data' -> (self jsonDataFrom: {'title' -> 'new test task'. 'explanation' -> 'This is a test task.'. 'timeEstimated' -> 42})).
	
	self should: testProject tasks size = (tasksCount + 1).
	
	newTask := testProject tasks detect:
		[:task | task id = (response at: 'value') asURI pathComponents last asNumber].
	
	self should: (response at: 'value') = ('/projects/{1}/tasks/{2}' format: {testProject id. newTask id}).
	self should: newTask title = 'new test task'.
	self should: newTask explanation = 'This is a test task.'.
	self should: newTask timeEstimated = 42.
	self should: newTask project = testProject! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/18/2009 15:36'!
testPostAttachDeveloperToTask
	"Try to attach testUser to testTask via http POST request. Verify this by looking into the database directly."
	
	| response developersCount |
	developersCount := testTask developers size.
	response := self
		post: ('projects/{1}/tasks/{2}/attachdeveloper' format: {testProject id. testTask id})
		fields: (Dictionary with: 'data' -> (self jsonDataFrom: {'args' -> {'/users/{1}' format: {testUser id}}})).
	
	self should: testTask developers size = (developersCount + 1).
	self should: (testTask developers includes: testUser).
	self should: (response at: 'value') = ('/users/{1}' format: {testUser id})! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/18/2009 15:10'!
testPutOneProject
	"Try to update information for a project via http PUT request. This operation should be idempotent. Verify this by looking into the database directly."
	
	| projectCount response developersCount tasksCount userStoriesCount |
	projectCount := XpfProject all size.
	userStoriesCount := testProject userStories size.
	developersCount := testProject developers size.
	tasksCount := testProject tasks size.
	
	2 timesRepeat: [	
		response := self
			put: 'projects/', testProject id asString
			fields: (Dictionary with: 'data' -> (self jsonDataFrom: {'title' -> 'New Title'})).
			
		self should: response statusCode = 200.
		self should: XpfProject all size = projectCount.
		self should: testProject title = 'New Title'.
		
		self should: testProject userStories size = userStoriesCount.
		self should: testProject developers size = developersCount.
		self should: testProject tasks size = tasksCount]! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/18/2009 15:11'!
testPutOneTask
	"Try to update information for a task of testProject via http PUT request. This operation should be idempotent. Verify this by looking into the database directly."
	
	| taskCount response developersCount userStoriesCount |
	taskCount := testProject tasks size.
	userStoriesCount := testProject userStories size.
	developersCount := testProject developers size.
	
	2 timesRepeat: [
		response := self
			put: ('projects/{1}/tasks/{2}' format: {testProject id. testTask id})
			fields: (Dictionary with: 'data' -> (self jsonDataFrom: {'title' -> 'New Title'. 'explanation' -> 'New Explanation'. 'timeEstimated' -> 25. 'timeActual' -> 50})).
			
		self should: response statusCode = 200.
		self should: testProject tasks size = taskCount.
		self should: testTask title = 'New Title'.
		self should: testTask explanation = 'New Explanation'.
		self should: testTask timeEstimated = 25.
		self should: testTask timeActual = 50.
		
		self should: testTask userStories size = userStoriesCount.
		self should: testTask developers size = developersCount.
		self should: testTask project = testProject]! !

!RestResourceTest methodsFor: 'tests' stamp: 'mt 2/18/2009 14:59'!
testRequestChain
	"Check whether request chains are created correctly."
	
	| pc1 pc2 pc3 rc1 rc2 rc3|
	pc1 := {'projects'. '23'. 'userstories'. '7'. 'tasks'. '12'} asOrderedCollection.
	pc2 := {'projects'. '23'. 'userstories'} asOrderedCollection.
	pc3 := {'this'. 'is'. 'invalid'} asOrderedCollection.
	
	rc1 := RestXpfResourceHttpHandler new getRequestChainFor: pc1.
	rc2 := RestXpfResourceHttpHandler new getRequestChainFor: pc2.
	rc3 := RestXpfResourceHttpHandler new getRequestChainFor: pc3.
	
	self should: rc1 first = (XpfProject -> 23).
	self should: rc1 second = (#userStories -> 7).
	self should: rc1 third = (#tasks -> 12).
	
	self should: rc2 first = (XpfProject -> 23).
	self should: rc2 second = (#userStories -> nil).
	
	self should: rc3 isEmpty.! !

!RestTest class methodsFor: 'testing' stamp: 'cs 2/15/2009 14:14'!
isAbstract
	^ self name = 'RestTest'! !

!RestTest methodsFor: 'helping' stamp: 'mt 2/18/2009 16:17'!
createHttpResponseFrom: aString

	| response contentLength |
	response := HttpResponse new.
	aString linesDo: [:line|
		(line startsWith: 'HTTP')
			ifTrue: [response status: (HttpResponse statusSymbolFor: (line findTokens: Character space) second asNumber)].
		(line startsWith: 'Content-type')
			ifTrue: [response contentType: (line findTokens: $:) second withBlanksTrimmed].
		(line startsWith: 'Content-length')
			ifTrue: [contentLength := (line findTokens: $:) second asInteger]].
		
	response content: (aString copyFrom: (aString size - contentLength) to: aString size).

	^response! !

!RestTest methodsFor: 'helping' stamp: 'mt 2/18/2009 16:15'!
delete: aPath
	"Send an HTTP request with DELETE to localhost at the port of the (singleton) rest HTTP service."
	
	^ self createHttpResponseFrom: (HTTPSocket httpDelete: ('http://localhost:{1}/{2}' format: {RestHttpService port. aPath}))! !

!RestTest methodsFor: 'helping' stamp: 'mt 2/18/2009 16:51'!
get: aPath
	"Send an HTTP request with GET to localhost at the port of the (singleton) rest HTTP service."

	^ self createHttpResponseFrom: (HTTPSocket httpGet: ('http://localhost:{1}/{2}' format: {RestHttpService port. aPath}))! !

!RestTest methodsFor: 'helping' stamp: 'mt 2/18/2009 15:09'!
jsonDataFrom: someFields
	"Create a string for all fields in the dictionary in JSON."
	
	| stream |
	stream := TextStream with: ''.
	(Dictionary newFrom: someFields) jsonWriteOn: stream.	
	^ stream contents! !

!RestTest methodsFor: 'helping' stamp: 'mt 2/18/2009 16:57'!
post: aPath fields: aDictionary
	"Send an HTTP request with POST to localhost at the port of the (singleton) rest HTTP service. Append the given fields to the request body."
	
	^ self createHttpResponseFrom: (HTTPSocket
		httpPost: ('http://localhost:{1}/{2}' format: {RestHttpService port. aPath})
		args:	(aDictionary keysAndValuesCollect: [:key :value | key -> {value}])
		accept: '*/*'
	) content! !

!RestTest methodsFor: 'helping' stamp: 'mt 2/18/2009 14:16'!
put: aPath fields: aDictionary
	"Send an HTTP request with PUT to localhost at the port of the (singleton) rest HTTP service. Append the given fields to the request body."
	
	^ HTTPSocket
		httpPut:	('http://localhost:{1}/{2}' format: {RestHttpService port. aPath})
		args:	(aDictionary keysAndValuesCollect: [:key :value | key -> {value}])! !

!RestTest methodsFor: 'running' stamp: 'mt 2/16/2009 15:05'!
setUp
	RestHttpService start.
	"wait until the service is up"
	(Delay forMilliseconds: 100) wait.! !

!RestTest methodsFor: 'running' stamp: 'cs 2/15/2009 14:27'!
tearDown
	RestHttpService stop.! !
